@@ -1,4 +1,4 @@
-# $Id: TEA.pm,v 0.96 2001/03/25 19:26:31 ams Exp $
+# $Id: TEA.pm,v 0.99 2001/03/28 16:36:13 ams Exp $
 # Copyright 2001 Abhijit Menon-Sen <ams@wiw.org>
 
 package Crypt::TEA;
@@ -9,20 +9,20 @@ use DynaLoader;
 use vars qw( @ISA $VERSION );
 
 @ISA = qw( DynaLoader );
-($VERSION) = q$Revision: 0.96 $ =~ /(\d+\.\d+)/;
+($VERSION) = q$Revision: 0.99 $ =~ /(\d+\.\d+)/;
 
 bootstrap Crypt::TEA $VERSION;
 
-sub blocksize {  8; }
-sub keysize   { 16; }
+use constant keysize   => 16;
+use constant blocksize => 8;
 
 sub new
 {
     my ($class, $key, $rounds) = @_;
 
-    $rounds ||= 32;
     croak "Usage: ".__PACKAGE__."->new(\$key [, \$rounds])" unless $key;
-    return bless {ks => $key, rounds => $rounds}, ref($class) || $class;
+
+    return Crypt::TEA::setup($key, $rounds || 32);
 }
 
 sub encrypt
@@ -30,7 +30,7 @@ sub encrypt
     my ($self, $data) = @_;
 
     croak "Usage: \$tea->encrypt(\$data)" unless ref($self) && $data;
-    Crypt::TEA::crypt($data, $data, $self->{'ks'}, $self->{'rounds'}, 0);
+    $self->crypt($data, $data, 0);
 }
 
 sub decrypt
@@ -38,7 +38,7 @@ sub decrypt
     my ($self, $data) = @_;
 
     croak "Usage: \$tea->decrypt(\$data)" unless ref($self) && $data;
-    Crypt::TEA::crypt($data, $data, $self->{'ks'}, $self->{'rounds'}, 1);
+    $self->crypt($data, $data, 1);
 }
 
 1;
@@ -61,17 +61,15 @@ $plaintext  = $tea->decrypt($ciphertext);
 
 =head1 DESCRIPTION
 
-This module implements TEA encryption, as described by David J. Wheeler
-and Roger M. Needham in
-<http://www.ftp.cl.cam.ac.uk/ftp/papers/djw-rmn/djw-rmn-tea.html>.
-
 TEA is a 64-bit symmetric block cipher with a 128-bit key and a variable
 number of rounds (32 is recommended). It has a low setup time, and
 depends on a large number of rounds for security, rather than a complex
-algorithm.
+algorithm. It was developed by David J. Wheeler and Roger M. Needham,
+and is described at
+<http://www.ftp.cl.cam.ac.uk/ftp/papers/djw-rmn/djw-rmn-tea.html>.
 
-The module supports the Crypt::CBC interface, with the following
-functions.
+This module implements TEA encryption. It supports the Crypt::CBC
+interface, with the following functions.
 
 =head2 Functions
 
@@ -87,18 +85,18 @@ Returns the size (in bytes) of the key (16, in this case).
 
 =item new($key, $rounds)
 
-This creates a new Crypt::TEA object with the specified key (assumed to
-be of keysize() bytes). The optional rounds parameter specifies the
-number of rounds of encryption to perform, and defaults to 32.
+This creates a new Crypt::TEA object with the specified key. The
+optional rounds parameter specifies the number of rounds of encryption
+to perform, and defaults to 32.
 
 =item encrypt($data)
 
-Encrypts $data (of blocksize() bytes) and returns the corresponding
+Encrypts blocksize() bytes of $data and returns the corresponding
 ciphertext.
 
 =item decrypt($data)
 
-Decrypts $data (of blocksize() bytes) and returns the corresponding
+Decrypts blocksize() bytes of $data and returns the corresponding
 plaintext.
 
 =back
